function [Ftotal, Fviscous] = computeForce(p, e, t, u, nu, edgeIds)

% computeForce - Compute force acting from fluid on given surfaces.
%
% This QuickerSim CFD Toolbox function calculates both total and viscous
% force acting from fluid on the boundary given by edgeIds.
%
% [Ftotal, Fviscous] = computeForce(p, e, t, u, nu, edgeIds);
%
% Input arguments:
% p, e, t - arrays which store computational mesh (all details of  the PET
%     mesh format described in help for importMeshGmsh function).
% u - a column vector with 2*nVnodes elements (nVnodes stands for global
%     number of velocity unknowns in the mesh - refer to help to
%     convertMeshToSecondOrder for details) which stores velocity field.
% nu - kinematic viscosity value in one of two formats:
%     1. a scalar value constant in the whole domain,
%     2. a vector of nu values for each node in the mesh.
% edgeIds - row vector of ids of edges forming the boundary on which the
%     force is to be computed.
% 
% Output arguments:
% Ftotal - a two-element row vector of the resulting force (both x and y
%     component on the first and second entry respectively).
% Fviscous - a two-element row vector of x and y component of the viscous
%     force term. Hence, the net pressure part of force can be computed as
%     Ftotal-Fviscous.
%
% Examples:
%       Compute force acting on wall composed from edges 5 and 7.
%       F = computeForce(p, e, t, u, nu, [5 7]);
%
% Visit also www.quickersim.com/cfd-toolbox-for-matlab/index or simply
% www.quickersim.com homepage, get e-mail support from developers team at
% support@quickersim.com or buy support service for your business.
% Check out also a list of currently on-going workshops and trainings with
% many additional examples and documentation.
% We also develop customized apps and offer on-site training for your team.
%
% See also: BOUNDARYINTEGRAL2D, BOUNDARYFLUX2D, COMPUTEMOMENT,
%           IMPORTMESHGMSH.

% Compute pressure force
pressure = generatePressureData(u, p, t);
Fpressure = boundaryIntegral2D(p,e,pressure,edgeIds,'normalVector');
%disp(Fpressure);

% Compute viscous force
nnodes = size(p,2);
ugrad = solutionGradient2D(p,t,u(1:nnodes));
vgrad = solutionGradient2D(p,t,u(nnodes+1:2*nnodes));

if(length(nu)==1)
    nu = repmat(nu,nnodes,1);
end

f1 = [-nu.*ugrad(:,1); -nu.*ugrad(:,2)];
f2 = [-nu.*vgrad(:,1); -nu.*vgrad(:,2)];

fx_viscous = boundaryFlux2D(p,e,f1,edgeIds);
fy_viscous = boundaryFlux2D(p,e,f2,edgeIds);

Fviscous = [fx_viscous fy_viscous];
Ftotal = Fpressure + Fviscous;

end