function [Mtotal, Mviscous] = computeMoment(p, e, t, u, nu, edgeIds, aboutPoint)

% computeMoment - Compute moment of force acting on given faces.
%
% This QuickerSim CFD Toolbox function calculates both total and viscous
% part of the force moment acting from fluid on the boundary given by
% edgeIds.
%
% [Mtotal, Mviscous] = computeMoment(p, e, t, u, nu, edgeIds, aboutPoint);
%
% Input arguments:
% p, e, t - arrays which store computational mesh (all details of  the PET
%     mesh format described in help for importMeshGmsh function).
% u - a column vector with 2*nVnodes elements (nVnodes stands for global
%     number of velocity unknowns in the mesh - refer to help to
%     convertMeshToSecondOrder for details) which stores velocity field.
% nu - kinematic viscosity value in one of two formats:
%     1. a scalar value constant in the whole domain,
%     2. a vector of nu values for each node in the mesh.
% edgeIds - row vector of ids of edges forming the boundary on which the
%     force is to be computed.
% aboutPoint - a two-element row vector of x and y coordinate of the point
%     about which the moment value should be calculated.
% 
% Output arguments:
% Mtotal - value of the total moment resulting from both pressure and
%     viscous forces.
% Mviscous - value of the moment generated by the viscous part of forces.
%     Hence, the moment value resulting from only pressure forces can be
%     computed as Mtotal-Mviscous.
%
% Examples:
%       Calculate moment acting on wall composed from edges 5 and 7 about
%       point P = (2.5, 0.5)
%       M = computeMoment(p, e, t, u, nu, [5 7], [2.5 0.5]);
%
% Visit also www.quickersim.com/cfd-toolbox-for-matlab/index and subscribe.
% You will get access to documentation, code examples, video and pdf
% tutorials. You can also ask for simple help at support@quickersim.com or
% buy dedicated, professional support service for your business.
% Check out also a list of currently on-going workshops and trainings with
% many additional examples, documentation and heat transfer applications.
% We also develop customized apps and offer on-site training for your team.
%
% See also: BOUNDARYINTEGRAL2D, BOUNDARYFLUX2D, COMPUTEFORCE,
%           EXTRACTDATAALONGLINE, EXTRACTNODEIDSONEDGES, IMPORTMESHGMSH.

% Compute pressure force
pressure = generatePressureData(u, p, t);
ry = p(2,:)'-aboutPoint(2);
rx = p(1,:)'-aboutPoint(1);
momentField = [-pressure.*ry; pressure.*rx];
Mpressure = boundaryFlux2D(p,e,momentField,edgeIds);
%disp(Mpressure);

% Compute viscous force
nnodes = size(p,2);
ugrad = solutionGradient2D(p,t,u(1:nnodes));
vgrad = solutionGradient2D(p,t,u(nnodes+1:2*nnodes));

if(length(nu)==1)
    nu = repmat(nu,nnodes,1);
end

f1 = [nu.*ugrad(:,1).*ry; nu.*ugrad(:,2).*ry];
f2 = [-nu.*vgrad(:,1).*rx; -nu.*vgrad(:,2).*rx];

%norm(vgrad(:,2))
%mean(vgrad(:,2))
%vgrad(:,2)

Mx_viscous = boundaryFlux2D(p,e,f1,edgeIds);
My_viscous = boundaryFlux2D(p,e,f2,edgeIds);

Mviscous = Mx_viscous + My_viscous;
Mtotal = Mpressure + Mviscous;

end