function [p, e, t, nVnodes, nPnodes, indices] = convertMeshToSecondOrder(p, e, t)

% convertMeshToSecondOrder - Convert 2-D linear grid to second order grid
%
% This QuickerSim CFD Toolbox function converts a first order linear mesh
% read from an external file (e.g. gmsh etc.) to the second order mesh
% which is needed for fluid flow simulation with P2-P1 elements (so called
% Taylor-Hood pair) with second order shape functions for velocity
% discretization and linear shape functions for pressure approximation.
%
% [p, e, t, nVnodes, nPnodes, indices] = convertMeshToSecondOrder(p, e, t)
%
% Input arguments:
% p - array of node coordinates
% e - array defining boundaries of the domain
% t - array of triangular elements
%
% Output arguments:
% p, e, t - arrays defining mesh - see help for importMeshGmsh function for
%     detailed description of mesh PET format.
% p - as above with additional second order nodes in the middle of each
%     edge of every element
% e - as above with added indices of new nodes. For a linear 2-D mesh this
%     array has 7 rows, for a second order grid the number of rows is increased
%     to 8 - the last row stores the indices of the new nodes added in the
%     middle of each edge
% t - as above. For a 2-D linear mesh this array stores indices of element
%     nodes in rows 1 to 3 and the id of the domain in the last 4-th row. In
%     case of second order grid the corner nodes of each triangle are still
%     stored in rows 1 to 3 and rows 4 to 6 store correspondingly indices of
%     new second order nodes in the following order: node 4 (placed between
%     original 1st and 2nd node), node 5 (placed between original 2nd and 3rd
%     node) and node 6 (placed between original 3rd and 1st node).
% nVnodes - total number of nodes associated with velocity unknowns (so called
%     velocity nodes - i.e. all nodes in the second order mesh).
% nPnodes - total number of nodes associated with pressure unknowns (so
%     called pressure nodes - i.e. all nodes of the original linear grid).
% indices - a structure containing for each node indices of the x-velocity,
%     y-velocity and pressure unknown in the global solution vector u.
%
% Visit also www.quickersim.com/cfd-toolbox-for-matlab/index and subscribe.
% You will get access to documentation, code examples, video and pdf
% tutorials. You can also ask for simple help at support@quickersim.com or
% buy dedicated, professional support service for your business.
% Check out also a list of currently on-going workshops and trainings with
% many additional examples, documentation and heat transfer applications.
% We also develop customized apps and offer on-site training for your team.
%
% See also IMPORTMESHGMSH, GENERATEINDICES2D.

nnodes = size(p,2);
nelements = size(t,2);

Edges = cell(nnodes,1);
AddNodes = cell(nnodes,1);

for i = 1:nnodes
    Edges{i} = i;
    AddNodes{i} = i;
end

t = [t; zeros(3, nelements)];
t(7,:) = t(4,:);
el = nnodes+1;

for i = 1:nelements
    for j = 1:3
        n1 = t(j,i);
        n2 = t(mod(j,3)+1,i);
        nloc = sort([n1 n2]);
        n1 = nloc(1);
        n2 = nloc(2);
        
        ind = find(Edges{n1}==n2);
        
        if(ind)
            t(3+j,i) = AddNodes{n1}(ind);
        else
            Edges{n1}(end+1) = n2;
            AddNodes{n1}(end+1) = el;
            t(3+j,i) = el;
            el = el+1;
        end
    end
end

el = el-1;

przyrost = el-nnodes;
nnodesfull = el;

p = [p zeros(2,przyrost)];

for i = 1:nelements
    for j = 1:3
        n1 = t(j,i);
        n2 = t(mod(j,3)+1,i);
        
        p(:,t(j+3,i)) = 0.5*(p(:,n1)+p(:,n2));
    end
end

% Enrich edges
nEdges = size(e,2);
e = [e; zeros(1,nEdges)];

for i = 1:nEdges
    n1 = e(1,i);
    n2 = e(2,i);
    nloc = sort([n1 n2]);
    n1 = nloc(1);
    n2 = nloc(2);
    ind = find(Edges{n1}==n2);
    e(8,i) = AddNodes{n1}(ind);
end

nVnodes = size(p,2);
nPnodes = max(max(t(1:3,:)));
indices = generateIndices2D(p,t);

end