function nodeIds = extractNodeIdsOnEdges(e, edgeIds)

% extractNodeIdsOnEdges - Extract ids of nodes lying on given edges.
%
% This QuickerSim CFD Toolbox function is intended for convenient
% preparation of plots of solution along boundaries of the domain. It
% allows to quickly find ids of the nodes which are lying on the given
% edges. Later this ids may be used to generate e.g. a plot of pressure
% along certain wall.
%
% nodeIds = extractNodeIdsOnEdges(e, edgeIds)
%
% Input arguments:
% e - array of edge elements as defined in help to the importMeshGmsh
%     function.
% edgeIds - a row vector of edgeIds for which the node ids are to be
%     generated (e.g. ids of edges along which we later want to draw a
%     solution plot).
%
% Output arguments:
% nodeIds - global ids of mesh nodes lying on the edges specified by
%     edgeIds.
%
% Examples:
%       Draw pressure plot along edge 5, 6 and 7.
%           pressure = generatePressureData(u, p, t);
%           wallNodes = extractNodeIdsOnEdges(e, [5 6 7]);
%           scatter(p(1,wallNodes), pressure(wallNodes)')
%
% Visit also www.quickersim.com/cfd-toolbox-for-matlab/index and subscribe.
% You will get access to documentation, code examples, video and pdf
% tutorials. You can also ask for simple help at support@quickersim.com or
% buy dedicated, professional support service for your business.
% Check out also a list of currently on-going workshops and trainings with
% many additional examples, documentation and heat transfer applications.
% We also develop customized apps and offer on-site training for your team.
%
% See also: DRAWSTREAMLINES, EXTRACTDATAALONGLINE, GENERATEPRESSUREDATA, 
%           IMPORTMESHGMSH.

nnodes = max(max(e(1:2,:)));
nodeIds = zeros(1,nnodes);
    
    for faceId = edgeIds
        for edge = 1:size(e,2)
            if(e(5,edge)==faceId)
                dnodes = e([1 2 8],edge);
                
                nodeIds(dnodes) = 1;
            end
        end
    end
    
nodeIds = find(nodeIds);

end